import os
import platform

UBUNTU_UPSTART_JOB_FILE = \
"""# {tty} - getty
#
# This service maintains a getty on {tty} from the point the system is
# started until it is shut down again.

start on runlevel [23]
stop on runlevel [!23]

respawn
exec /sbin/getty -8 38400 -n -i -l {path} {tty}
"""


def install(tty_dev, greeter_path):
    dist, version, nick = platform.dist()
    if dist == 'Ubuntu':
        major, minor = (int(i) for i in version.split('.'))
        if major >= 10:
            install_on_ubuntu(tty_dev, greeter_path)
            return
    raise NotImplementedError("{} {} not supported".format(dist, version))


def move_to_backup(path):
    backup_path = path + ".before_greeter"
    if not os.path.exists(backup_path):
        os.rename(path, backup_path)


def install_on_ubuntu(tty_dev, greeter_path):
    template = UBUNTU_UPSTART_JOB_FILE.format(tty=tty_dev, path=greeter_path)
    job_path = "/etc/init/{tty_dev}.conf".format(tty_dev=tty_dev)
    move_to_backup(job_path)
    with open(job_path, "w") as f:
        f.write(template)



__import__("pkg_resources").declare_namespace(__name__)

__all__ = ['greet', 'Greeter', 'install']

import signal
from .greeter import Greeter
from .install import install

def greet(product, version, log_path, status_program, login_program):
    signal.signal(signal.SIGHUP, signal.SIG_IGN)
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    greeter = Greeter(product, version, log_path, status_program, login_program)
    greeter.run()



__version__ = "0.0.2"
__git_commiter_name__ = "Tal Yalon"
__git_commiter_email__ = "yalon@infinidat.com"
__git_branch__ = '(detached from v0.0.2)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '5d6d819b4a41ab8b8556b036501b580e19268b81'
__git_head_subject__ = 'added install greeter on ubuntu'
__git_head_message__ = ''
__git_dirty_diff__ = ''


import os
import traceback
import shlex
from subprocess import Popen, PIPE
import npyscreen


# We override ActionForm to provide us Login & Refresh buttons (Login is the OK button, Refresh is the Cancel button)
class MainForm(npyscreen.ActionForm):
    OK_BUTTON_TEXT = ' Login '
    CANCEL_BUTTON_TEXT = ' Refresh '
    CANCEL_BUTTON_BR_OFFSET = (2, 16)

    def __init__(self, *args, **kwargs):
        super(MainForm, self).__init__(*args, **kwargs)
        self.prefix_errors = []

    def on_ok(self):
        error = self.parentApp.login()
        # If we reached here it means we failed to execute the login program.
        self.prefix_errors = ["Failed to execute login program:"] + error
        self.parentApp.switchForm("MAIN")

    def on_cancel(self):
        self.parentApp.switchForm("MAIN")

    def beforeEditing(self):
        status, lines = self.parentApp.get_form_data(self.prefix_errors)
        self.prefix_errors = []

        status_widget = self.get_widget('status')
        status_widget.entry_widget.color = 'GOOD' if status else 'DANGER'
        status_widget.value = 'OK' if status else 'ERROR'

        log_widget = self.get_widget('log')
        log_widget.hidden = status
        log_widget.values = lines


class Greeter(npyscreen.NPSAppManaged):
    def __init__(self, product_name, version, log_file_path, status_program, login_program):
        super(Greeter, self).__init__()
        self.product_name = product_name
        self.version = version
        self.log_file_path = log_file_path
        self.status_program = status_program
        self.login_program = login_program

    def load_log(self):
        """returns list of log lines"""
        try:
            with open(self.log_file_path, 'r') as f:
                return [s.rstrip() for s in f.readlines()]
        except:
            return traceback.format_exc().split("\n")

    def get_status(self):
        """returns (bool, list of error lines)"""
        try:
            args = shlex.split(self.status_program)
            with open(os.devnull, 'w') as devnull:
                p = Popen(args, stdin=devnull, stdout=PIPE, stderr=PIPE, close_fds=True, shell=False)
                stdout, stderr = p.communicate()
                exitcode = p.wait()
            return exitcode == 0, ["Status command: " + repr(args), "Status stdout:"] + stdout.split("\n") + ["Status stderr:"] + stderr.split("\n")
        except:
            return False, ["Error executing status program:"] + traceback.format_exc().split("\n")

    def clear_and_restore_terminal(self):
        # We want to clear the screen and restore the terminal mode before execing the login program
        import curses
        self.getForm('MAIN').erase()
        curses.curs_set(1)
        curses.reset_shell_mode()

    def close_fds(self):
        import glob
        import sys
        fd_paths = glob.glob("/proc/{}/fd/*".format(str(os.getpid())))
        fds = set(int(os.path.basename(p)) for p in fd_paths)
        exclude_fds = set(f.fileno() for f in (sys.stdin, sys.stdout, sys.stderr))
        for fd in (fds - exclude_fds):
            try:
                os.close(fd)
            except:
                pass

    def login(self):
        """this method should never return, but if it does it returns a list of error lines"""
        try:
            self.clear_and_restore_terminal()
            parts = shlex.split(self.login_program)
            self.close_fds()
            os.execv(parts[0], parts)
        except:
            return traceback.format_exc().split("\n")

    def create_form(self):
        f = MainForm(name="MAIN", framed=False, parentApp=self)
        f.add(npyscreen.FixedText, value=self.product_name + ' ' + self.version, editable=False)
        f.add(npyscreen.FixedText, value='', editable=False)  #  spacer
        f.add(npyscreen.TitleText, w_id='status', name='Service Status:', labelColor='DEFAULT', editable=False, use_two_lines=False)
        f.add(npyscreen.FixedText, value='', editable=False)  #  spacer
        f.add(npyscreen.BoxTitle, w_id='log', name='Status & Log Output', color='DEFAULT', scroll_exit=True)
        return f

    def get_form_data(self, prefix_errors=[]):
        status, status_output = self.get_status()
        lines = prefix_errors + status_output + ([ "", "Log file:"] + self.load_log() if not status else [])
        return status, lines

    def onStart(self):
        form = self.create_form()
        self.registerForm("MAIN", form)




import os
import click
import traceback
from infi.traceback import traceback_context


@click.command()
@click.argument('product')
@click.argument('version')
@click.argument('log_path')
@click.argument('status_program')
@click.argument('login_program')
@click.option('--terminfo', default='/lib/terminfo', help='terminfo directory')
def greet(product, version, log_path, status_program, login_program, terminfo):
    """Runs a custom greeter screen.

    PRODUCT is the product name to display

    VERSION is the product version to display

    LOG_PATH is a path to an error log in case the service is not running

    STATUS_PROGRAM is a program with arguments to check if the service is running (exit 0 - running, otherwise not)

    LOGIN_PROGRAM is the program to exec when the user chooses LOGIN
    """
    with traceback_context():
        try:
            if 'TERMINFO' not in os.environ:
                os.putenv('TERMINFO', terminfo)
            from . import greet
            return greet(product, version, log_path, status_program, login_program)
        except:
            traceback.print_exc()
            os._exit(1)


@click.command()
@click.argument('tty_dev')
@click.argument('greeter_path')
def install(tty_dev, greeter_path):
    """Installs a greeter script for a specific tty.

    TTY_DEV is the tty device to install on (tty1, tty2, etc.).

    GREETER_PATH is a path to a greeter executable file (without arguments).
    """
    with traceback_context():
        try:
            from . import install
            install(tty_dev, greeter_path)
        except:
            traceback.print_exc()
            os._exit(1)
